import {
  csv,
  max,
  select,
  timeParse,
  scaleTime,
  scaleLinear,
  axisBottom,
  axisLeft,
  brushX,
  extent,
} from 'd3';


// Set Sizing Constants
const svg = select('svg');
const svgWidth = +svg.attr('width');
const svgHeight = +svg.attr('height');
const margin = { top: 20, right: 40, bottom: 120, left: 40 };
const margin2 = { top: 420, right: 40, bottom: 40, left: 40};
const width = svgWidth - margin.left - margin.right;
const height = svgHeight - margin.top - margin.bottom;
const height2 = svgHeight - margin2.top - margin2.bottom;


// Date Parser
const parseDate = timeParse('%b %Y');


// Set Scales and Axes
const x = scaleTime().range([0, width]);
const x2 = scaleTime().range([0, width]);
const y = scaleLinear().range([height, 0]);
const y2 = scaleLinear().range([height2, 0]);

const xAxis = axisBottom(x);
const xAxis2 = axisBottom(x2);
const yAxis = axisLeft(y);


// Set Brush
const brush = brushX()
  .extent([[0, 0], [width, height2]])
  .on('brush end', brushed);

// Set Area Functions
const area = d3.area()
  .curve(d3.curveMonotoneX)
  .x((d) => x(d.date))
  .y0(height)
  .y1((d) => y(d.price));
const area2 = d3.area()
  .curve(d3.curveMonotoneX)
  .x((d) => x2(d.date))
  .y0(height2)
  .y1((d) => y2(d.price));


// Create Focus and Context Groups
const focus = svg.append('g')
  .attr('class', 'focus')
  .attr('transform', `translate(${margin.left}, ${margin.top})`);
const context = svg.append('g')
  .attr('class', 'context')
  .attr('transform', `translate(${margin2.left}, ${margin2.top})`);


// Clip Off Brush-Zoom Overflow
svg.append('defs').append('clipPath')
    .attr('id', 'clip')
  .append('rect')
    .attr('width', width)
    .attr('height', height);


csv('data.csv', type).then((data) => {
  
  // Set Axes Domains
  x.domain(extent(data, (d) => d.date));
  y.domain([0, max(data, (d) => d.price)]);
  x2.domain(x.domain());
  y2.domain(y.domain());
  
  // Set Focus Axes and Path
  focus.append('path') // Data
    .datum(data)
    .attr('class', 'area')
    .attr('d', area);
  focus.append('g') // X-Axis
    .attr('class', 'axis axis--x')
    .attr('transform', `translate(0, ${height})`)
    .call(xAxis);
  focus.append('g') // Y-Axis
    .attr('class', 'axis axis--y')
    .call(yAxis);
  
  // Set Context Axes and Path
  context.append('path') // Data
    .datum(data)
    .attr('class', 'area')
    .attr('d', area2);
  context.append('g') // X-Axis
    .attr('class', 'axis axis--x')
    .attr('transform', `translate(0, ${height2})`)
    .call(xAxis2);
  context.append('g') // Y-Axis
    .attr('class', 'brush')
    .call(brush)
    .call(brush.move, x.range());
});


function brushed() {
  let s = d3.event.selection || x2.range();
  x.domain(s.map(x2.invert, x2));
  focus.select('.area').attr('d', area);
  focus.select('.axis--x').call(xAxis);
}


function type(d) {
  d.date = parseDate(d.date);
  d.price = +d.price;
  return d;
}











